@@ -16,6 +16,8 @@ from conversionkit import chainPostConverters, Conversion, set_error, Field
 from stringconvert import unicodeToUnicode, unicodeToBoolean
 from stringconvert.email import listOfEmails, unicodeToEmail
 from recordconvert import toRecord
+from databasepipe.setup import BaseSetupCmd
+from pipestack.ensure import ensure_method_bag, ensure_self_marble
 
 to_unicode = unicodeToUnicode()
 
@@ -25,7 +27,7 @@ class ErrorReportMarble(Marble):
     def report(marble, exc_info=None):
         if not hasattr(marble.bag, marble.aliases.mail):
             marble.bag.add(marble.aliases.mail)
-        # Use the mail bag to send a report
+        # Use the mail marble to send a report
         data = dict(
             to=marble.config.to,
             from_email=marble.config.from_email,
@@ -71,11 +73,137 @@ class ErrorReportMarble(Marble):
             pass
         if not errordocument_available:
             # Otherwise fall back to a simple message.
-            marble.bag.http.response.status = '500 Internal Server Error'
-            marble.bag.http.response.headers = [
-                ('Content-type', 'text/plain; charset=utf-8')
+            marble.bag.http_response.status = '500 Internal Server Error'
+            marble.bag.http_response.header_list = [
+                dict(name='Content-type', value='text/plain; charset=utf-8')
             ]
-            marble.bag.http.response.body = ['An error occurred']
+            marble.bag.http_response.body = ['An error occurred']
+
+
+    #def on_set_flow_state(marble, flow_state):
+    #    marble.__dict__['exceptions'] = []
+
+    @ensure_self_marble('database', 'ticket')
+    def add(marble, exc_info):
+        # Try to assemble the current URL:
+        import urlconvert
+        url = urlconvert.extract_url(marble.bag)
+        database = marble.bag[marble.aliases['database']]
+        ticket = marble.bag[marble.aliases['ticket']]
+        #marble.excpetions.append(AttributeDict(type=type, message=message))
+        type=repr(exc_info[0])
+        message=str(exc_info[1])
+        try:
+            html = marble.display_html(exc_info)
+        except:
+            html = 'Failed to generate an HTML report'
+        database.insert_record(
+            'errorreport_exception',
+            dict(
+                user=ticket.get('username'),
+                url=url,
+                date=datetime.datetime.now().strftime('%Y-%m-%d %H:%M'),
+                type=type,
+                message=message,
+                html=html,
+            ),
+            'uid',
+        )
+        # Make sure this change is commiteed
+        database._connection.commit()
+        log.debug('ErrorReport pipe added an exception type %s: %s', type, message)
+
+    @ensure_self_marble('database')
+    def get(marble, uid):
+        database = marble.bag[marble.aliases['database']]
+        rows = database.query(
+            '''
+            SELECT 
+                html
+            FROM errorreport_exception
+            where uid = ?
+            ''',
+            (uid,),
+            format='list'
+        )
+        if not rows:
+            return None
+        else:
+            return rows[0][0]
+
+    @ensure_self_marble('database')
+    def hide(marble, uid):
+        database = marble.bag[marble.aliases['database']]
+        rows = database.query(
+            '''
+            UPDATE errorreport_exception
+            SET new = 0
+            WHERE
+              uid = ?
+            ''',
+            (uid,),
+            fetch=False,
+        )
+
+    @ensure_self_marble('database')
+    def show(marble, uid):
+        database = marble.bag[marble.aliases['database']]
+        rows = database.query(
+            '''
+            UPDATE errorreport_exception
+            SET new = 1
+            WHERE
+              uid = ?
+            ''',
+            (uid,),
+            fetch=False,
+        )
+
+    @ensure_self_marble('database')
+    def all(marble, hide_old=True):
+        database = marble.bag[marble.aliases['database']]
+        values=[]
+        sql = '''
+            SELECT 
+              uid     
+            , date    
+            , message 
+            , type    
+            , user    
+            , url     
+            , new
+            FROM errorreport_exception
+        '''
+        if not hide_old:
+            sql += '''
+            WHERE new=1
+            '''
+        sql += '''
+            ORDER BY url DESC, type DESC, message DESC, user DESC
+        '''
+        rows = database.query(
+            sql,
+            tuple(values),
+        )
+        warning_records = {}
+        for row in rows:
+            key = (
+                row['url'],
+                row['message'],
+                row['type'],
+                row['user'], 
+            )
+            value = {
+                'uid': row['uid'], 'date': row['date'], 'new': row['new']
+            }
+            if warning_records.has_key(key):
+                warning_records[key].append(value)
+            else:
+                warning_records[key] = [value]
+        data = warning_records.items()
+        data.sort()
+        return data
+
 
 def productionTo():
     def productionTo_post_converter(conversion, state=None):
@@ -85,13 +213,11 @@ def productionTo():
 
 class ErrorReportPipe(MarblePipe):
     marble_class = ErrorReportMarble
-    #required_section_error_message = "'%(name)s.to', '%(name)s.from_email' and '%(name)s.subject'"
-    default_aliases = AttributeDict(mail='mail', errordocument='errordocument')
+    default_aliases = AttributeDict(mail='mail', errordocument='errordocument', database='database', ticket='ticket')
     options = dict(
         subject = Field(to_unicode),
         to = Field(
             listOfEmails(),
-            #"The option '%(name)s.to' cannot be empty",
         ),
         from_email = Field(unicodeToEmail()),
         from_name = Field(to_unicode),
@@ -99,6 +225,10 @@ class ErrorReportPipe(MarblePipe):
             unicodeToBoolean(),
             missing_or_empty_default = False,
         ),
+        enable_database_report = Field(
+            unicodeToBoolean(),
+            missing_or_empty_default = False,
+        ),
         enable_email_report = Field(
             unicodeToBoolean(),
             missing_or_empty_default = False,
@@ -126,6 +256,8 @@ class ErrorReportPipe(MarblePipe):
             return marble.error_document() 
         log.info('Error occurred, reporting it. Error: %r', original_error[1])
         # Try to generate an error report:
+        if marble.config.enable_database_report:
+            marble.add(original_error)
         try: 
             msg = marble.display_html(original_error)
             type = 'html'
@@ -152,11 +284,14 @@ class ErrorReportPipe(MarblePipe):
                 raise
         # Now either display that report or a 500 error document
         if marble.config.enable_traceback:
-            bag.http.response.status = '500 Internal Server Error'
-            bag.http.response.headers = [
-                ('Content-type', 'text/%s; charset=utf-8'%(type == 'html' and 'html' or 'plain'))
+            bag.http_response.status = '500 Internal Server Error'
+            bag.http_response.header_list = [
+                dict(
+                    name='Content-type', 
+                    value='text/%s; charset=utf-8'%(type == 'html' and 'html' or 'plain')
+                )
             ]
-            bag.http.response.body = [msg]
+            bag.http_response.body = [msg]
         else:
             marble.error_document()
         # Now email that report if requested to do so
@@ -169,3 +304,33 @@ class ErrorReportPipe(MarblePipe):
                     raise
         bag.interrupt_flow(error_handled = True)
 
+class SetupCmd(BaseSetupCmd):
+    table_names_string = 'error report tables'
+
+    @ensure_method_bag('database')
+    def create_tables(self, bag):
+        bag[self.aliases['database']].query(
+            '''
+            CREATE TABLE errorreport_exception (
+                uid INTEGER PRIMARY KEY
+              , date TEXT NOT NULL
+              , message TEXT NOT NULL
+              , type TEXT
+              , user TEXT
+              , url TEXT
+              , html TEXT
+              , new BOOLEAN DEFAULT 1;
+            );
+            ''',
+            fetch=False
+        )
+        
+    @ensure_method_bag('database')
+    def drop_tables(self, bag):
+        bag[self.aliases['database']].query(
+            '''
+            DROP TABLE IF EXISTS errorreport_exception;
+            ''',
+            fetch=False,
+        )
+
